/*
 * Copyright 2023 Endless OS Foundation LLC
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gtk/gtk.h>

#include "gis-util.h"

void
gis_add_style_from_resource (const char *resource_path)
{
  g_autoptr(GtkCssProvider) provider = gtk_css_provider_new ();

  gtk_css_provider_load_from_resource (provider, resource_path);
  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
                                              GTK_STYLE_PROVIDER (provider),
                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
}

gboolean
gis_kernel_command_line_has_argument (const char *arguments[])
{
  GError *error = NULL;
  g_autofree char *contents = NULL;
  g_autoptr (GString) pattern = NULL;
  gboolean has_argument = FALSE;
  size_t i;

  if (!g_file_get_contents ("/proc/cmdline", &contents, NULL, &error)) {
    g_error_free (error);
    return FALSE;
  }

  /* Build up the pattern by iterating through the alternatives,
   * escaping all dots so they don't match any character but period,
   * and adding word boundary specifiers around the arguments so
   * substrings don't get matched.
   *
   * Also, add a | between each alternative.
   */
  pattern = g_string_new (NULL);
  for (i = 0; arguments[i] != NULL; i++) {
    g_autofree char *escaped_argument = g_regex_escape_string (arguments[i], -1);

    if (i > 0) {
      g_string_append (pattern, "|");
    }

    g_string_append (pattern, "\\b");

    g_string_append (pattern, escaped_argument);

    g_string_append (pattern, "\\b");
  }

  has_argument = g_regex_match_simple (pattern->str, contents, 0, 0);

  return has_argument;
}

void
gis_substitute_word_in_text (char **text,
                             const char *old_word,
                             const char *new_word)
{
  g_autoptr (GError) error = NULL;
  g_autofree char *pattern = g_strdup_printf ("\\b%s\\b", old_word);
  g_autoptr (GRegex) regex = g_regex_new (pattern, 0, 0, &error);
  g_autofree char *replacement_text = NULL;

  if (text == NULL || *text == NULL) {
    return;
  }

  if (error != NULL) {
    g_debug ("Error creating regex to match %s: %s\n", old_word, error->message);
  }

  replacement_text = g_regex_replace (regex, *text, -1, 0, new_word, 0, &error);

  if (error != NULL) {
    g_debug ("Error replacing %s with %s in %s: %s\n", old_word, new_word, *text, error->message);
    return;
  }

  g_free (*text);
  *text = g_steal_pointer (&replacement_text);
}
